########################################################################
import numpy as np
from modele_donnees import *
from modele_cesar import *
import re


rep_trav = './'

########################################################################

def charRL(Eps,LX,LY,pairesGD,pairesBH,K_penal) :

# forces nodales sur les ddl des elements RL
# On calcule E : (x2-x1) , on multiplie par k et on charge les deux noeuds opposes
    listnod=[]
    fnodX=[] ; fnodY=[]
    deltaM = np.array ( [ LX , 0. ] )
    force = np.dot ( Eps, deltaM )
    fx= K_penal*force[0] ; fy=K_penal*force[1]

    for paire in pairesGD :
        listnod.extend( [ paire[2] , paire[1] ] )
        fnodX.extend( [fx,-fx ] )
        fnodY.extend( [fy,-fy ] )
        
    gX_H=GEN_SOL(IGEN = 2, NP = 2*len(pairesGD) , NUM= listnod , IL = 1, F= fnodX )
    gY_H=GEN_SOL(IGEN = 2, NP = 2*len(pairesGD) , NUM= listnod , IL = 2, F= fnodY )

    listnod=[] 
    fnodX=[] ; fnodY=[]
    deltaM = np.array ( [ 0. , LY ] )
    force = np.dot ( Eps, deltaM )
    fx= K_penal*force[0] ; fy=K_penal*force[1]

    for paire in pairesBH :
        listnod.extend( [ paire[2] , paire[1] ] )
        fnodX.extend( [fx,-fx ] )
        fnodY.extend( [fy,-fy ] )
        
    gX_V=GEN_SOL(IGEN = 2, NP = 2*len(pairesBH) , NUM= listnod , IL = 1, F= fnodX )
    gY_V=GEN_SOL(IGEN = 2, NP = 2*len(pairesBH) , NUM= listnod , IL = 2, F= fnodY )

    sol_RL = CHAR_SOL(M1=0, GEN= [ gX_H, gY_H, gX_V, gY_V ] )

    return CHAR (M=4, OPT=  sol_RL )
########################################################################

fic_dom = rep_trav + 'cellule.geo'
fic_mail = rep_trav + 'cellule.msh'

########################################################################

# parametres geometriques 
LX  = 1. ; LY  = 1.  ; R   = 0.2
d   = 0.1       # longueur des elements sur le contour de l inclusion
d2  = 0.025     # longueur des elements sur le contour de la cellule de base

########################################################################

# definition de la geometrie
op = [ 
# definition des points
      POINT(1,      0.,   0.,  0., d),  POINT(2, LX,  0.,  0., d),  
      POINT(3,      LX,   LY,  0., d),  POINT(4, 0.,  LY,  0., d),  
      POINT(5, LX/2.-R, LY/2,  0., d2),        
      POINT(6,   LX/2., LY/2., 0., d2),  
      POINT(7, LX/2.+R, LY/2., 0., d2),  

# definition des lignes
      LIGNE( 1, 1, 2), LIGNE(2, 2, 3), 
      LIGNE(3, 3, 4), LIGNE( 4, 4, 1), 
      CERCLE (5, P1= 5, PC=6, P2=7),
      CERCLE (6, P1= 7, PC=6, P2=5),
      
# definition des contours des zones du maillage
      BOUCLE(1, 'LIGNE', [1 , 2, 3, 4, 6, 5]), 
      SURFACE(1, [1]), 
      
      BOUCLE(2, 'LIGNE', [5, 6]), 
      SURFACE(2, [2]), 

# definition de deux blocs surfaciques
      BLOC('1', 'SURFACE', [1]),
      BLOC('2', 'SURFACE', [2]),
      
# et de quatre blocs-lignes pour les bords gauche/droit et haut/bas
# bloc 3 = base / bloc 4 = droit / bloc 5 = haut / bloc 6 = gauche
      BLOC('3', 'LIGNE', [1]),
      BLOC('4', 'LIGNE', [2]),
      BLOC('5', 'LIGNE', [3]),
      BLOC('6', 'LIGNE', [4]),

# option : elements quadratiques
      OPTION(MAIL_ORDRE_ELEM = 2, MAIL_ORDRE_2_INCOMPLET = 1),
        ]
   
dom = DOMAINE(NOM = 'dom',
              DIM = 2,
              FIC_OUT = FICHIER(NOM = fic_dom, FORMAT = 'GMSH'),
              OPER = op,
              VISU = 'NON')    

dom.mailler(FIC_OUT = FICHIER(NOM = fic_mail, FORMAT = 'GMSH'))

########################################################################

# on recupere le maillage dans le fichier .msh genere par GMSH

mail = MAIL(FIC = fic_mail, FMT = 'GMSH', NDIM = 2, 
            FAMI= { '1':'MB', '2':'MB'},
            GF = [ '3', '4', '5', '6' ],
            )

# NB : GF donne acces aux groupes de facettes dans l'objet mail qui permettront
# de definir les bords

nnt = len(mail.vcorg)//mail.ndim
ndim = mail.ndim
nelt = len(mail.pnumel)-1

print("Pour info : "+str(nnt)+" noeuds / "+str(nelt)+" elements")

########################################################################

nb_nds_base   = mail.gn[3]['NP'] ; nds_base   = mail.gn[3]['NUM']
nb_nds_droite = mail.gn[4]['NP'] ; nds_droite = mail.gn[4]['NUM']
nb_nds_haut   = mail.gn[5]['NP'] ; nds_haut   = mail.gn[5]['NUM']
nb_nds_gauche = mail.gn[6]['NP'] ; nds_gauche = mail.gn[6]['NUM']

list_base = []
for noeud in nds_base :
    list_base.append ( [noeud,mail.vcorg[2*noeud-2]] )
list_base = sorted(list_base, key=lambda node : node[1])

list_haut = []
for noeud in nds_haut :
    list_haut.append ( [noeud,mail.vcorg[2*noeud-2]] )
list_haut = sorted(list_haut, key=lambda node : node[1])

pairesBH = []
index=0
for noeud in list_base:
    pairesBH.append( (list_base[index][1],list_base[index][0],list_haut[index][0]))
    index += 1

list_gauche = []
for noeud in nds_gauche :
    list_gauche.append ( [noeud,mail.vcorg[2*noeud-1]] )
list_gauche = sorted(list_gauche, key=lambda node : node[1])

list_droite = []
for noeud in nds_droite :
    list_droite.append ( [noeud,mail.vcorg[2*noeud-1]] )
list_droite = sorted(list_droite, key=lambda node : node[1])

pairesGD = []
index=0
for noeud in list_gauche :
    pairesGD.append( (list_gauche[index][1],list_gauche[index][0],list_droite[index][0]) )
    index += 1

########################################################################
# Definition des nouveaux elements

last_pnumel = mail.pnumel[-1]
mail.ngrpe += 2

for paire in pairesBH :
    nelt=nelt+2
    mail.pnumel=mail.pnumel+[last_pnumel+2, last_pnumel+4] 
    mail.numel=mail.numel+[int(paire[2]), int(paire[1])]
    mail.numel=mail.numel+[int(paire[2]), int(paire[1])]
    mail.type=mail.type+['RL  ','RL  ']
    mail.groupe=mail.groupe+[3,4]
    last_pnumel += 4
    
for paire in pairesGD :
    nelt=nelt+2
    mail.pnumel=mail.pnumel+[last_pnumel+2, last_pnumel+4] 
    mail.type=mail.type+['RL  ','RL  ']
    mail.groupe=mail.groupe+[3,4]
    mail.numel=mail.numel+[int(paire[2]), int(paire[1])]
    mail.numel=mail.numel+[int(paire[2]), int(paire[1])]
    last_pnumel += 4

print("Pour info : "+str(nnt)+" noeuds / "+str(nelt)+" elements")

########################################################################

# objet COOR
coor = COOR(M = 0,  
            M1 = 0,
            NNT = nnt,
            NDIM = ndim,
            VCORG = mail.vcorg)

########################################################################

# caracteristiques de la matrice et de l inclusion
Emat = 3000.  ; Pmat = 0.3
Einc = 50000. ; Pinc = 0.3
# coefficient de penalisation pour les RL
K_penal = 10000.*Einc

m_matrice = MB(NOMG = 'matrice',
                    ACTI = 'A',
                    IMOD = 1,
                    CARA = MB_ELI( RO = 0., YOUNG = Emat , POISS = Pmat, INAT = 1 )
                    )

m_inclusion = MB(NOMG = 'inclusion',
                    ACTI = 'A',
                    IMOD = 1,
                    CARA = MB_ELI( RO = 0., YOUNG = Einc , POISS = Pinc, INAT = 1 )
                    )

m_RLx = RL(NOMG = 'RLx',
                    ACTI = 'A',
                    IDL = [1, 1], C = [1., -1.], P = K_penal 
                    )
                    
m_RLy = RL(NOMG = 'RLy',
                    ACTI = 'A',
                    IDL = [2, 2], C = [1., -1.], P = K_penal 
                    )            
# objet ELEM
elem = ELEM(M = 0, 
            M1 = 0,
            NELT = nelt, 
            NGRPE = mail.ngrpe,
            PNUMEL = mail.pnumel, 
            NUMEL = mail.numel,
            TYPE = mail.type, 
            GROUPE = mail.groupe,
            GRPES = [ m_matrice, m_inclusion, m_RLx , m_RLy ])

########################################################################
# Conditions aux limites : 
# la solution est definie a un mouvement de corps rigide pres
# on bloque en x le noeud situe au milieu du bord gauche
# et en y le noeud situe au milieu de bord inferieur

for paire in pairesBH :
    coorY=paire[0]
    if abs(float(coorY)-LY/2) < d2/2 :
        blocageY= int(paire[1])

for paire in pairesGD :
    coorX=paire[0]
    if abs(float(coorX)-LX/2) < d2/2 :
        blocageX= int(paire[1])

cond = COND(M = 2,
            OPT = [COND_NUL(GEN = [GEN_NUL(IGEN = 2,
                                           NP = 1,
                                           NUM = [blocageX], 
                                           IDL = [1, 0],
                                           NDLN = 2) ] ),
                   COND_NUL(GEN = [GEN_NUL(IGEN = 2,
                                           NP = 1,
                                           NUM = [blocageY], 
                                           IDL = [0, 1],
                                           NDLN = 2) ] ),
                                           ])  

########################################################################

# Tenseurs de deformation macro
EpsXX = np.array( [ [ 1. , 0. ], [ 0., 0. ] ] )
EpsYY = np.array( [ [ 0. , 0. ], [ 0., 1. ] ] )
EpsXY = np.array( [ [ 0. , 1. ], [ 1., 0. ] ] )

char_XX = charRL (EpsXX,LX,LY,pairesGD,pairesBH,K_penal)
char_YY = charRL (EpsYY,LX,LY,pairesGD,pairesBH,K_penal)
char_XY = charRL (EpsXY,LX,LY,pairesGD,pairesBH,K_penal)

########################################################################
# on fait un calcul mecanique non lineaire (MCNL)
# methode des contraintes initiales (IMET=1), 500 iterations maxi
# tolérance de 0.001
# on applique le chargement en un increment VCT = [ 1.]
# on utilise les options suivantes MUL (solveur multifrontal), 
# DTO (affichage des déformations totales) et HPE (sorties specifiques 
# des moyennes des contraintes dans le fichier .list)

mul = MUL (MOD = 'MCNL', IGND = 1 , IIO = 0 )
dto = DTO (MOD = 'MCNL')
hpe = HPE (MOD = 'MCNL')
mcnl = MCNL (M = 0, 
              NINCR = 1, NITER = 500, TOL = 0.001, IMET = 1,
              VCT = [ 1.], 
              OPT = [ mul, dto, hpe  ]       ) 

#
# constitution du jeu de donnees et lancement du calcul

expo = EXPO (IPGMSH = 1)

fic1_pos   = rep_trav + 'cellule_M1.pos'
fic1_list  = rep_trav + 'cellule_M1.list'
exe_XX = EXEC(MSH  = fic1_pos , LIST = fic1_list)   
jeu_XX = JDD(EXEC = exe_XX,
          EXPO = expo,  
          COOR = coor,
          ELEM = elem,
          COND = cond,
          CHAR = char_XX,
          MCNL = mcnl
          )

fic2_pos   = rep_trav + 'cellule_M2.pos'
fic2_list  = rep_trav + 'cellule_M2.list'
exe_YY = EXEC(MSH  = fic2_pos , LIST = fic2_list)
jeu_YY = JDD(EXEC = exe_YY,
          EXPO = expo,  
          COOR = coor,
          ELEM = elem,
          COND = cond,
          CHAR = char_YY,
          MCNL = mcnl
          )

fic3_pos   = rep_trav + 'cellule_M3.pos'
fic3_list  = rep_trav + 'cellule_M3.list'
exe_XY = EXEC(MSH  = fic3_pos , LIST = fic3_list)
jeu_XY = JDD(EXEC = exe_XY,
          EXPO = expo,  
          COOR = coor,
          ELEM = elem,
          COND = cond,
          CHAR = char_XY ,
          MCNL = mcnl
          )

jeu_XX.lancer()
jeu_YY.lancer()
jeu_XY.lancer()
   
#####################################################################################################
# recherche des moyennes des contraintes dans le fichier .list

print("")
print("****************************************************************************")
print("************************ SYNTHESE DES RESULTATS :  *************************")
print("********************  MODULES ELASTIQUES HOMOGENEISES **********************")
print("****************************************************************************")
print("")

f = open(fic1_list,"r")
for line in f :
    test =re.search(r'.*MOYENNE DES CONTRAINTES.*',line)
    if(str(test) !='None'):
        rep=f.readline().split()
        CXXXX=f"{ float(rep[0]):12.5e}  "
        CYYXX=f"{ float(rep[1]):12.5e}  "
        CXYXX=f"{ float(rep[2]):12.5e}  "
        CZZXX=f"{ float(rep[3]):12.5e}  "
        print("cas 1 : CXXXX,CYYXX,CXYXX,CZZXX = ",CXXXX,CYYXX,CXYXX,CZZXX) 
f.close()
 
f = open(fic2_list,"r")
for line in f :
    test =re.search(r'.*MOYENNE DES CONTRAINTES.*',line)
    if(str(test) !='None'):
        print("")
        rep=f.readline().split()
        CXXYY=f"{ float(rep[0]):12.5e}  "
        CYYYY=f"{ float(rep[1]):12.5e}  "
        CXYYY=f"{ float(rep[2]):12.5e}  "
        CZZYY=f"{ float(rep[3]):12.5e}  "
        print("cas 2 : CXXYY,CYYYY,CXYYY,CZZYY = ",CXXYY,CYYYY,CXYYY,CZZYY) 
f.close()

f = open(fic3_list,"r")
for line in f :
    test =re.search(r'.*MOYENNE DES CONTRAINTES.*',line)
    if(str(test) !='None'):
        print("")
        rep=f.readline().split()
        CXXXY=f"{ float(rep[0])/2.:12.5e}  "
        CYYXY=f"{ float(rep[1])/2.:12.5e}  "
        CXYXY=f"{ float(rep[2])/2.:12.5e}  "
        CZZXY=f"{ float(rep[3])/2.:12.5e}  "
        print("cas 3 : CXXXY,CYYXY,CXYXY,CZZXY = ",CXXXY,CYYXY,CXYXY,CZZXY) 
f.close()





